@@ -172,7 +172,9 @@ packets = {
     8: Struct("health",
         UBInt16("hp"),
     ),
-    9: Struct("respawn"),
+    9: Struct("respawn",
+        UBInt8("unknown"),
+    ),
     10: grounded,
     11: Struct("position", position, grounded),
     12: Struct("orientation", orientation, grounded),
@@ -269,6 +271,14 @@ packets = {
         SBInt32("x"),
         SBInt32("y"),
         SBInt32("z"),
+        SBInt32("unknown1"),
+        If(lambda context: context["unknown1"] > 0,
+            Embed(Struct("more_unknowns",
+                UBInt16("unknown2"),
+                UBInt16("unknown3"),
+                UBInt16("unknown4"),
+            )),
+        ),
     ),
     24: Struct("mob",
         UBInt32("eid"),
@@ -412,8 +422,26 @@ packets = {
         UBInt32("count"),
         MetaField("unknown5", lambda context: context["count"] * 3),
     ),
-    70: Struct("unknown-0x46",
-        UBInt8("unknown1"),
+    61: Struct("unknown-0x3d",
+        UBInt32("unknown1"),
+        UBInt32("unknown2"),
+        UBInt8("unknown3"),
+        UBInt32("unknown4"),
+        UBInt32("unknown5"),
+    ),
+    70: Struct("state",
+        Enum(UBInt8("state"),
+            bad_bed=0,
+            start_rain=1,
+            stop_rain=2,
+        ),
+    ),
+    71: Struct("thunderbolt",
+        UBInt32("eid"),
+        UBInt8("unknown"),
+        SBInt32("x"),
+        SBInt32("y"),
+        SBInt32("z"),
     ),
     100: Struct("window-open",
         UBInt8("wid"),
@@ -466,6 +494,15 @@ packets = {
         AlphaString("line3"),
         AlphaString("line4"),
     ),
+    131: Struct("unknown-0x83",
+        UBInt16("unknown1"),
+        UBInt16("unknown2"),
+        PascalString("data", length_field=UBInt8("length")),
+    ),
+    200: Struct("statistics",
+        UBInt32("sid"), # XXX I could be an Enum
+        UBInt8("count"),
+    ),
     255: Struct("error",
         AlphaString("message"),
     ),
