@@ -1,3 +1,4 @@
+from types import ModuleType
 from xml.sax import saxutils
 
 from exocet import ExclusiveMapper, getModule, load, pep302Mapper
@@ -10,6 +11,7 @@ from zope.interface.exceptions import BrokenImplementation
 from zope.interface.exceptions import BrokenMethodImplementation
 from zope.interface.verify import verifyObject
 
+from bravo.errors import PluginException
 from bravo.ibravo import InvariantException, ISortedPlugin
 
 blacklisted = set([
@@ -31,11 +33,6 @@ overrides = {
 bravoMapper = ExclusiveMapper(pep302Mapper,
                               blacklisted).withOverrides(overrides)
 
-class PluginException(Exception):
-    """
-    Signal an error encountered during plugin handling.
-    """
-
 def sort_plugins(plugins):
     """
     Make a sorted list of plugins by dependency.
@@ -142,23 +139,52 @@ def verify_plugin(interface, plugin):
 
     raise PluginException("Plugin failed verification")
 
-def get_plugins(interface, package):
+def synthesize_parameters(parameters):
+    """
+    Create a faked module which has the given parameters in it.
+
+    This should work everywhere. If it doesn't, let me know.
+    """
+
+    module = ModuleType("parameters")
+    module.__dict__.update(parameters)
+    return module
+
+def get_plugins(interface, package, parameters=None):
     """
     Lazily find objects in a package which implement a given interface.
 
     The objects must also implement ``twisted.plugin.IPlugin``.
 
+    If the optional dictionary of parameters is provided, it will be passed
+    into each plugin module as the "bravo.parameters" module. An example
+    access from inside the plugin:
+
+    >>> from bravo.parameters import foo, bar
+
+    Since the parameters are available as a real module, the parameters may be
+    imported and used like any other module:
+
+    >>> from bravo import parameters as params
+
     This is a rewrite of Twisted's ``twisted.plugin.getPlugins`` which uses
     Exocet instead of Twisted to find the plugins.
 
     :param interface interface: the interface to match against
     :param str package: the name of the package to search
+    :param dict parameters: parameters to pass into the plugins
     """
 
+    mapper = bravoMapper
+
+    if parameters:
+        mapper = mapper.withOverrides(
+            {"bravo.parameters": synthesize_parameters(parameters)})
+
     p = getModule(package)
     for pm in p.iterModules():
         try:
-            m = load(pm, bravoMapper)
+            m = load(pm, mapper)
             for obj in vars(m).itervalues():
                 try:
                     adapted = IPlugin(obj, None)
@@ -171,19 +197,7 @@ def get_plugins(interface, package):
         except ImportError, ie:
             log.msg(ie)
 
-__plugin_cache = {}
-
-def clear_plugin_cache():
-    """
-    Clear the plugin cache.
-
-    This has the (intended) side effect of causing all plugins to be reloaded
-    on the next retrieval.
-    """
-
-    __plugin_cache.clear()
-
-def retrieve_plugins(interface, cached=True):
+def retrieve_plugins(interface, parameters=None):
     """
     Look up all plugins for a certain interface.
 
@@ -191,18 +205,15 @@ def retrieve_plugins(interface, cached=True):
     plugins from disk or discover new plugins.
 
     :param interface interface: the interface to use
-    :param bool cached: whether to use the in-memory plugin cache
+    :param dict parameters: parameters to pass into the plugins
 
     :returns: a dict of plugins, keyed by name
     :raises PluginException: no plugins could be found for the given interface
     """
 
-    if cached and interface in __plugin_cache:
-        return __plugin_cache[interface]
-
     log.msg("Discovering %s..." % interface)
     d = {}
-    for p in get_plugins(interface, "bravo.plugins"):
+    for p in get_plugins(interface, "bravo.plugins", parameters):
         try:
             verify_plugin(interface, p)
             d[p.name] = p
@@ -213,10 +224,9 @@ def retrieve_plugins(interface, cached=True):
         # Sortable plugins need their edges mirrored.
         d = add_plugin_edges(d)
 
-    __plugin_cache[interface] = d
     return d
 
-def retrieve_named_plugins(interface, names):
+def retrieve_named_plugins(interface, names, parameters=None):
     """
     Look up a list of plugins by name.
 
@@ -224,12 +234,13 @@ def retrieve_named_plugins(interface, names):
 
     :param interface interface: the interface to use
     :param list names: plugins to find
+    :param dict parameters: parameters to pass into the plugins
 
     :returns: a list of plugins
     :raises PluginException: no plugins could be found for the given interface
     """
 
-    d = retrieve_plugins(interface)
+    d = retrieve_plugins(interface, parameters)
 
     # Handle wildcards and options.
     names = expand_names(d, names)
@@ -238,14 +249,16 @@ def retrieve_named_plugins(interface, names):
         return [d[name] for name in names]
     except KeyError, e:
         raise PluginException("Couldn't find plugin %s for interface %s!" %
-            (e.args[0], interface))
+            (e.args[0], interface.__name__))
 
-def retrieve_sorted_plugins(interface, names):
+def retrieve_sorted_plugins(interface, names, parameters=None):
     """
     Look up a list of plugins, sorted by interdependencies.
+
+    :param dict parameters: parameters to pass into the plugins
     """
 
-    l = retrieve_named_plugins(interface, names)
+    l = retrieve_named_plugins(interface, names, parameters)
     try:
         return sort_plugins(l)
     except KeyError, e:
