@@ -3,9 +3,11 @@ from zope.interface import implements
 
 from bravo.blocks import blocks, items
 from bravo.entity import Chest, Sign
-from bravo.ibravo import IBuildHook
+from bravo.ibravo import IPreBuildHook
 from bravo.utilities.coords import split_coords
 
+from bravo.parameters import factory
+
 class Tile(object):
     """
     Place tiles.
@@ -13,10 +15,10 @@ class Tile(object):
     You almost certainly want to enable this plugin.
     """
 
-    implements(IBuildHook)
+    implements(IPreBuildHook)
 
     @inlineCallbacks
-    def build_hook(self, factory, player, builddata):
+    def pre_build_hook(self, player, builddata):
         item, metadata, x, y, z, face = builddata
 
         if item.slot == items["sign"].slot:
@@ -73,7 +75,7 @@ class Tile(object):
 
             bigx, smallx, bigz, smallz = split_coords(x, z)
 
-            # Not much to do, just tell the chunk about this chest.
+           # Not much to do, just tell the chunk about this chest.
             chunk = yield factory.world.request_chunk(bigx, bigz)
             c = Chest(smallx, y, smallz)
             chunk.tiles[smallx, y, smallz] = c
@@ -85,64 +87,6 @@ class Tile(object):
     before = tuple()
     after = ("build",)
 
-class Build(object):
-    """
-    Place a block in a given location.
-
-    You almost certainly want to enable this plugin.
-    """
-
-    implements(IBuildHook)
-
-    def build_hook(self, factory, player, builddata):
-        block, metadata, x, y, z, face = builddata
-
-        # Don't place items as blocks.
-        if block.slot not in blocks:
-            return True, builddata
-
-        # Check for orientable blocks.
-        if not metadata and block.orientable():
-            metadata = block.orientation(face)
-            if metadata is None:
-                # Oh, I guess we can't even place the block on this face.
-                return True, builddata
-
-        # Make sure we can remove it from the inventory first.
-        if not player.inventory.consume((block.slot, 0), player.equipped):
-            # Okay, first one was a bust; maybe we can consume the related
-            # block for dropping instead?
-            if not player.inventory.consume((block.drop, 0), player.equipped):
-                return True, builddata
-
-        # Offset coords according to face.
-        if face == "-x":
-            x -= 1
-        elif face == "+x":
-            x += 1
-        elif face == "-y":
-            y -= 1
-        elif face == "+y":
-            y += 1
-        elif face == "-z":
-            z -= 1
-        elif face == "+z":
-            z += 1
-
-        builddata = builddata._replace(x=x, y=y, z=z, face="noop")
-
-        # Set the block and data. Don't bother collecting the Deferreds.
-        factory.world.set_block((x, y, z), block.slot)
-        if metadata:
-            factory.world.set_metadata((x, y, z), metadata)
-
-        return True, builddata
-
-    name = "build"
-
-    before = tuple()
-    after = tuple()
-
 class BuildSnow(object):
     """
     Makes building on snow behave correctly.
@@ -150,17 +94,21 @@ class BuildSnow(object):
     You almost certainly want to enable this plugin.
     """
 
-    implements(IBuildHook)
+    implements(IPreBuildHook)
 
-    @inlineCallbacks
-    def build_hook(self, factory, player, builddata):
-        block = yield factory.world.get_block((builddata.x, builddata.y, builddata.z))
+    def pre_build_hook(self, player, builddata):
+        d = factory.world.get_block((builddata.x, builddata.y, builddata.z))
 
-        if block == blocks["snow"].slot:
-            # Building any block on snow causes snow to get replaced.
-            builddata = builddata._replace(face="+y", y=builddata.y - 1)
+        @d.addCallback
+        def adjust_block(block):
+            if block == blocks["snow"].slot:
+                # Building any block on snow causes snow to get replaced.
+                bd = builddata._replace(face="+y", y=builddata.y - 1)
+            else:
+                bd = builddata
+            return True, bd
 
-        returnValue((True, builddata))
+        return d
 
     name = "build_snow"
 
@@ -168,5 +116,4 @@ class BuildSnow(object):
     after = ("build",)
 
 tile = Tile()
-build = Build()
 build_snow = BuildSnow()
