@@ -1,12 +1,14 @@
 from zope.interface import implements
 
-from bravo.ibravo import IConsoleCommand
+from bravo.ibravo import IConsoleCommand, IChatCommand
+
+from bravo.parameters import factory
 
 class Meliae(object):
 
     implements(IConsoleCommand)
 
-    def console_command(self, factory, parameters):
+    def console_command(self, parameters):
         out = "".join(parameters)
         try:
             import meliae.scanner
@@ -27,7 +29,7 @@ class Status(object):
 
     implements(IConsoleCommand)
 
-    def console_command(self, factory, parameters):
+    def console_command(self, parameters):
         protocol_count = len(factory.protocols)
         yield "%d protocols connected" % protocol_count
 
@@ -46,5 +48,22 @@ class Status(object):
     usage = ""
     info = "Print a quick summary of the server's status"
 
+class Colors(object):
+
+    implements(IChatCommand)
+
+    def chat_command(self, username, parameters):
+        from bravo.utilities.chat import chat_colors
+        names = """black dblue dgreen dcyan dred dmagenta dorange gray dgray
+        blue green cyan red magenta yellow""".split()
+        for pair in zip(chat_colors, names):
+            yield "%s%s" % pair
+
+    name = "colors"
+    aliases = tuple()
+    usage = ""
+    info = "Print the colors"
+
 meliae = Meliae()
 status = Status()
+colors = Colors()
