@@ -14,7 +14,7 @@ from twisted.python import log
 from bravo.chunk import Chunk
 from bravo.config import configuration
 from bravo.entity import Player
-from bravo.errors import SerializerReadException
+from bravo.errors import ChunkNotLoaded, SerializerReadException
 from bravo.ibravo import ISerializer, ISerializerFactory
 from bravo.plugin import (retrieve_named_plugins, verify_plugin,
     PluginException)
@@ -34,13 +34,35 @@ def coords_to_chunk(f):
         bigx, smallx, bigz, smallz = split_coords(x, z)
         d = self.request_chunk(bigx, bigz)
 
+        @d.addCallback
         def cb(chunk):
             return f(self, chunk, (smallx, y, smallz), *args, **kwargs)
-        d.addCallback(cb)
+
         return d
 
     return decorated
 
+def sync_coords_to_chunk(f):
+    """
+    Either get a chunk for the coordinates, or raise an exception.
+    """
+
+    @wraps(f)
+    def decorated(self, coords, *args, **kwargs):
+        x, y, z = coords
+
+        bigx, smallx, bigz, smallz = split_coords(x, z)
+        if (bigx, bigz) in self.chunk_cache:
+            chunk = self.chunk_cache[bigx, bigz]
+        elif (bigx, bigz) in self.dirty_chunk_cache:
+            chunk = self.dirty_chunk_cache[bigx, bigz]
+        else:
+            raise ChunkNotLoaded("Chunk (%d, %d) isn't loaded")
+
+        return f(self, chunk, (smallx, y, smallz), *args, **kwargs)
+
+    return decorated
+
 class World(object):
     """
     Object representing a world on disk.
@@ -78,16 +100,38 @@ class World(object):
     This cache is used to speed up logins near the spawn point.
     """
 
+    spawn = (0, 0, 0)
+    """
+    The spawn point.
+    """
+
+    time = 0
+    """
+    The current time.
+
+    This does not reflect the actual in-game time, just the time currently
+    saved in the level data.
+    """
+
     def __init__(self, name):
         """
-        Load a world from disk.
-
         :Parameters:
             name : str
                 The configuration key to use to look up configuration data.
         """
 
         self.config_name = "world %s" % name
+
+        self.chunk_cache = weakref.WeakValueDictionary()
+        self.dirty_chunk_cache = dict()
+
+        self._pending_chunks = dict()
+
+    def start(self):
+        """
+        Load a world from disk.
+        """
+
         world_url = configuration.get(self.config_name, "url")
         world_sf_name = configuration.get(self.config_name, "serializer")
 
@@ -98,34 +142,26 @@ class World(object):
             log.msg(pe)
             raise RuntimeError("Fatal error: Couldn't set up serializer!")
 
-        self.chunk_cache = weakref.WeakValueDictionary()
-        self.dirty_chunk_cache = dict()
-
-        self._pending_chunks = dict()
-
-        self.spawn = (0, 0, 0)
         self.seed = random.randint(0, sys.maxint)
-        self.time = 0
 
         # Check if we should offload chunk requests to ampoule.
         if configuration.getbooleandefault("bravo", "ampoule", False):
             try:
                 import ampoule
+                if ampoule:
+                    self.async = True
             except ImportError:
                 pass
-            else:
-                self.async = True
-
 
         # First, try loading the level, to see if there's any data out there
         # which we can use. If not, don't worry about it.
         d = maybeDeferred(self.serializer.load_level, self)
         d.addCallback(lambda chaff: log.msg("Loaded level data!"))
+        @d.addErrback
         def sre(failure):
             failure.trap(SerializerReadException)
             log.msg("Had issues loading level data...")
             log.msg(failure)
-        d.addErrback(sre)
 
         # And now save our level.
         if self.saving:
@@ -138,6 +174,30 @@ class World(object):
             (world_url, self.serializer.name))
         log.msg("Using Ampoule: %s" % self.async)
 
+    def stop(self):
+        """
+        Stop managing the world.
+
+        This can be a time-consuming, blocking operation, while the world's
+        data is serialized.
+
+        Note to callers: If you want the world time to be accurate, don't
+        forget to write it back before calling this method!
+        """
+
+        self.chunk_management_loop.stop()
+
+        # Flush all dirty chunks to disk.
+        for chunk in self.dirty_chunk_cache.itervalues():
+            self.save_chunk(chunk)
+
+        # Evict all chunks.
+        self.chunk_cache.clear()
+        self.dirty_chunk_cache.clear()
+
+        # Save the level data.
+        self.serializer.save_level(self)
+
     def enable_cache(self, size):
         """
         Set the permanent cache size.
@@ -221,6 +281,9 @@ class World(object):
     def postprocess_chunk(self, chunk):
         """
         Do a series of final steps to bring a chunk into the world.
+
+        This method might be called multiple times on a chunk, but it should
+        not be harmful to do so.
         """
 
         # Apply the current season to the chunk.
@@ -263,6 +326,7 @@ class World(object):
         if chunk.populated:
             self.chunk_cache[x, z] = chunk
             self.postprocess_chunk(chunk)
+            #self.factory.scan_chunk(chunk)
             returnValue(chunk)
 
         if self.async:
@@ -303,7 +367,10 @@ class World(object):
         # be done early becaues PendingEvents only fire exactly once and it
         # might fire immediately in certain cases.
         pe = PendingEvent()
+        # This one is for our return value.
         retval = pe.deferred()
+        # This one is for scanning the chunk for automatons.
+        #pe.deferred().addCallback(self.factory.scan_chunk)
         self._pending_chunks[x, z] = pe
 
         def pp(chunk):
@@ -424,3 +491,63 @@ class World(object):
         """
 
         chunk.dirty = True
+
+    @sync_coords_to_chunk
+    def sync_get_block(self, chunk, coords):
+        """
+        Get a block from an unknown chunk.
+
+        :returns: a ``Deferred`` with the requested value
+        """
+
+        return chunk.get_block(coords)
+
+    @sync_coords_to_chunk
+    def sync_set_block(self, chunk, coords, value):
+        """
+        Set a block in an unknown chunk.
+
+        :returns: a ``Deferred`` that will fire on completion
+        """
+
+        chunk.set_block(coords, value)
+
+    @sync_coords_to_chunk
+    def sync_get_metadata(self, chunk, coords):
+        """
+        Get a block's metadata from an unknown chunk.
+
+        :returns: a ``Deferred`` with the requested value
+        """
+
+        return chunk.get_metadata(coords)
+
+    @sync_coords_to_chunk
+    def sync_set_metadata(self, chunk, coords, value):
+        """
+        Set a block's metadata in an unknown chunk.
+
+        :returns: a ``Deferred`` that will fire on completion
+        """
+
+        chunk.set_metadata(coords, value)
+
+    @sync_coords_to_chunk
+    def sync_destroy(self, chunk, coords):
+        """
+        Destroy a block in an unknown chunk.
+
+        :returns: a ``Deferred`` that will fire on completion
+        """
+
+        chunk.destroy(coords)
+
+    @sync_coords_to_chunk
+    def sync_mark_dirty(self, chunk, coords):
+        """
+        Mark an unknown chunk dirty.
+
+        :returns: a ``Deferred`` that will fire on completion
+        """
+
+        chunk.dirty = True
