@@ -8,7 +8,10 @@ from types import ModuleType
 from zope.interface import Interface, implements
 
 DEBUG = False
-_sysModulesSpecialCases = {"os": ['path']}
+_sysModulesSpecialCases = {
+    "os": ['path'],
+    "twisted.internet": ["reactor"],
+}
 
 def trace(*args):
     if DEBUG:
@@ -520,7 +523,7 @@ class PackageContext(object):
                                                     self.mapper)
                     setattr(parent, seg, new)
                     parent = new
-            setattr(parent, modulePath[-1], initModule)
+                setattr(parent, modulePath[-1], initModule)
         self.allModules  = analyzePackage(self.packageMaker)
         sortedModules = robust_topological_sort(self.allModules)
         self.loadOrder = list(itertools.chain(*sortedModules))
